use actix_web::{get, web, HttpResponse, Responder};
use serde::Deserialize;
use sqlx::MySqlPool;

use crate::joint_err::JointErr;
use crate::spotify_get_top::get_top_tracks;
use crate::{spotify_auth, spotify_playlist, sql_tracks, sql_users};

#[derive(Deserialize)]
pub struct RefreshData {
    spotify_id: Option<String>,
    force: Option<bool>,
}

#[get("/refresh")]
pub async fn refresh_manager(
    data: web::Query<RefreshData>,
    state: web::Data<crate::state::AppState>,
) -> impl Responder {
    let pool = &state.pool;

    if let Some(spotify_id) = &data.spotify_id {
        let should_refresh = sql_users::get_refreshed_by_id(spotify_id, pool)
            .await
            .map(|time| sqlx::types::time::OffsetDateTime::now_utc() - time)
            .map(|since| since.whole_hours().abs() >= 6)
            .unwrap_or(false);
        if should_refresh || data.force.unwrap_or(false) {
            let e = refresh(spotify_id, pool).await;
            log::error!("Refresh res was:\n{:?}", e);
            HttpResponse::Ok().body("refreshing")
        } else {
            HttpResponse::TooManyRequests().body("it's been less than 6 hours. nah.")
        }
    } else {
        HttpResponse::BadRequest().body("there's no ID here idiot")
    }
}

#[allow(non_snake_case)]
#[get("/refresh_all")]
pub async fn refresh_all_manager(state: web::Data<crate::state::AppState>) -> impl Responder {
    let pool = &state.pool;
    let users = sqlx::query!(
        "
        SELECT SpotifyId FROM Users
        WHERE LastRefreshed < CURDATE() - INTERVAL 7 DAY
        "
    )
    .fetch_all(pool)
    .await;

    if let Ok(users) = users {
        for user in users {
            if let Err(e) = refresh(&user.SpotifyId, pool).await {
                log::error!(
                    "Tried to refresh for {}, failed with\n{:?}",
                    &user.SpotifyId,
                    e
                );
            }
        }
        HttpResponse::Ok().body("yeah ok")
    } else {
        HttpResponse::InternalServerError().body("where the users at")
    }
}

async fn refresh(spotify_id: &str, pool: &MySqlPool) -> Result<(), JointErr> {
    let token = spotify_auth::refresh_token_for_id(spotify_id, pool).await?;
    let internal_id = sql_users::get_internal_id_by_id(spotify_id, pool).await?;
    let top: Vec<_> = get_top_tracks(&token)
        .await?
        .items
        .iter()
        .map(|track| track.uri.clone())
        .collect();
    for track in top {
        log::info!("Trying to insert {}", track.0);
        sqlx::query!(
            "
            INSERT INTO Tracks (UserId, Uri, DateAdded)
            VALUES (?, ?, CURDATE())
            ON DUPLICATE KEY UPDATE DateAdded = CURDATE()
            ",
            internal_id,
            track.0
        )
        .execute(pool)
        .await?;
    }

    // Reset playlist
    set_playlist("6Zk0tvx4q6Zf5AZglvRLBT", pool).await?;

    Ok(())
}

async fn set_playlist(id: &str, pool: &MySqlPool) -> Result<(), JointErr> {
    let tracklist = sql_tracks::even_allocation(pool).await?;
    let emilia_id = sql_users::get_emilia_spotify_id(pool).await?;
    let token = spotify_auth::refresh_token_for_id(&emilia_id, pool).await?;

    spotify_playlist::set_playlist(id, tracklist, &token, pool).await?;

    Ok(())
}
