use sqlx::{mysql::MySqlQueryResult, MySqlPool};

use crate::spotify_auth::Token;

async fn add_user(
    email: &str,
    spotify_id: &str,
    refresh_token: &Token,
    pool: &MySqlPool,
) -> Result<MySqlQueryResult, sqlx::Error> {
    sqlx::query!(
        "INSERT INTO spotify.Users (Email, SpotifyId, RefreshToken)
        VALUES (?, ?, ?)",
        email.to_owned(),
        spotify_id.to_owned(),
        refresh_token.0.to_owned()
    )
    .execute(pool)
    .await
}

#[derive(Debug, PartialEq, Eq)]
pub enum AddUserSuccess {
    New,
    Duplicate,
}

pub async fn add_user_dont_care(
    email: &str,
    spotify_id: &str,
    refresh_token: &Token,
    pool: &MySqlPool,
) -> Result<AddUserSuccess, sqlx::Error> {
    let q = add_user(email, spotify_id, refresh_token, pool).await;
    match q {
        Ok(_) => Ok(AddUserSuccess::New),
        Err(sqlx::Error::Database(err)) => {
            if err.code().map(|e| e.into_owned()) == Some("23000".to_owned()) {
                Ok(AddUserSuccess::Duplicate)
            } else {
                Err(sqlx::Error::Database(err))
            }
        }
        Err(_) => Err(q.unwrap_err()),
    }
}

pub enum GetRefreshTokenByIdErr {
    SqlxError(sqlx::Error),
    Empty,
}
impl From<sqlx::Error> for GetRefreshTokenByIdErr {
    fn from(value: sqlx::Error) -> Self {
        GetRefreshTokenByIdErr::SqlxError(value)
    }
}

#[allow(non_snake_case)] // Fixes warning for macro
pub async fn get_refresh_token_by_id(
    spotify_id: &str,
    pool: &MySqlPool,
) -> Result<Token, GetRefreshTokenByIdErr> {
    let res = sqlx::query!(
        "SELECT RefreshToken FROM Users WHERE SpotifyId LIKE ?",
        spotify_id.to_owned()
    )
    .fetch_one(pool)
    .await?;
    match res.RefreshToken {
        Some(token) => Ok(Token(token)),
        None => Err(GetRefreshTokenByIdErr::Empty),
    }
}

#[allow(non_snake_case)] // Fixes warning for macro
pub async fn get_refreshed_by_id(
    spotify_id: &str,
    pool: &MySqlPool,
) -> Result<sqlx::types::time::OffsetDateTime, sqlx::Error> {
    let res = sqlx::query!(
        "SELECT LastRefreshed FROM Users WHERE SpotifyID LIKE ?",
        spotify_id.to_owned()
    )
    .fetch_one(pool)
    .await?;
    Ok(res.LastRefreshed)
}

#[allow(non_snake_case)] // Fixes warning for macro
pub async fn get_internal_id_by_id(spotify_id: &str, pool: &MySqlPool) -> Result<u32, sqlx::Error> {
    let res = sqlx::query!(
        "SELECT UserId FROM Users WHERE SpotifyID LIKE ?",
        spotify_id.to_owned()
    )
    .fetch_one(pool)
    .await?;
    Ok(res.UserId)
}

#[allow(non_snake_case)] // Fixes warning for macro
pub async fn get_emilia_spotify_id(pool: &MySqlPool) -> Result<String, sqlx::Error> {
    let res = sqlx::query!("SELECT SpotifyId FROM Users WHERE IsEmilia = 1")
        .fetch_one(pool)
        .await?;
    Ok(res.SpotifyId)
}
